//
// Created by Ari Trachtenberg on 9/10/24.
//

#ifndef PUBLIC_COMMON_H
#define PUBLIC_COMMON_H

#include <iostream>
#include <string>
#include <sstream>
using namespace std;

/**
 * A Singleton class of variables and functions that are common to multiple labs.
 */

class Common {
//
// Created by Ari Trachtenberg on 9/3/24, with support from ChatGPT
//

public:

// global variables
    static streambuf* orig_cin_buf;  // records the standard input buffer
    static streambuf* orig_cout_buf; // records the standard output buffer
    static istringstream inStream;   // input stream to produce from a given string
    static ostringstream outStream;  // captured output stream


// HELPERS

// Function to trim leading and trailing whitespace - from ChatGPT
    static string trim(const string& str) {
        size_t first = str.find_first_not_of(" \t\n\r\f\v");
        if (first == std::string::npos)
            return "";  // The string is all whitespace or empty

        size_t last = str.find_last_not_of(" \t\n\r\f\v");
        return str.substr(first, (last - first + 1));
    }

/**
 * Redirects input and output to strings
 * @param iStr The input string
 */
    static void redirIO(const string iStr) {
        // save the old buffers
        orig_cin_buf = cin.rdbuf();
        orig_cout_buf = cout.rdbuf();

        // set up streams
        inStream.str(iStr);
        outStream.str(""); outStream.clear();
        cin.rdbuf(inStream.rdbuf());
        cout.rdbuf(outStream.rdbuf());
    }

/**
 * Restore input and output to STDIN/STDOUT, respectively.
 * @requires redirIO() was called before this method
 * @return The output string that was captured since the redirect, trimmed of
 *         initial and final whitespace
 */
    static string restoreIO() {
        cin.rdbuf(orig_cin_buf);
        cout.rdbuf(orig_cout_buf);

        return trim(outStream.str());
    }
};
#endif //PUBLIC_COMMON_H
